<?php
/*
Plugin Name: ServiceU Events Calendar
Plugin URI: http://rinaldo.net/projects/serviceu-plugin
Description: Insert ServiceU.com events calendar using shortcode
Version: 1.6.0
Date: 29 December 2019
Author: Bery Rinaldo <bery@rinaldo.net>
Author URI: http://rinaldo.net/
*/

/*
	Copyright 2018-2019 Bery Rinaldo

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License, version 2, as 
	published by the Free Software Foundation. A copy of the license is at
	http://www.gnu.org/licenses/gpl-2.0.html

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
*/

/*
Note: This plugin requires WordPress version 4.3.1 or higher.

Information about the ServiceU Events Calendar plugin along with
instructions and examples can be found here:

http://rinaldo.net/projects/serviceu-plugin

[serviceu_events_calendar kind="all"] - The two month mini-calendar with all events
[serviceu_events_calendar kind="weekly"] - Today and the next 7 days
[serviceu_events_calendar kind="forsunday"] - from Sunday to next Sunday
[serviceu_events_calendar kind="thissundaydate"] - Sunday's date F j, Y format
[serviceu_events_calendar kind="todaysdate"] - today's date F j, Y format
[serviceu_events_calendar kind="+Women's Ministries"] - 1 month for a page
[serviceu_events_calendar kind="-Community Events"] - exclude these

You can only include one department name in the +/- syntax

Please excuse the use of globals, I know it is frowned upon, but this plugin is
essentially a port of code and it made use of globals, a signifincant amount of
effort will be needed to refactor that.  Room for version 2 or 3...
*/

#define( 'WP_DEBUG', true );

if( ! function_exists('bjrsu_serviceu_events_calendar') )
{
	#function bjrsu_serviceu_events_calendar_rewrite_rule()
	#{
		# This is very close - monthly?y=2019&m=7
		# but monthly/2019/07 does not
	
	#	add_rewrite_rule(
	#			'monthly/([0-9]+)/([0-9]+)/?$',
	#			'index.php?year=$matches[1]&month=$matches[2]',
	#			'top'
	#		);
    		#global $wp_rewrite;
    		#$plugin_url = plugins_url( 'month.php', __FILE__ );
    		#$plugin_url = substr( $plugin_url, strlen( home_url() ) + 1 );
    		// The pattern is prefixed with '^'
    		// The substitution is prefixed with the "home root", at least a '/'
    		// This is equivalent to appending it to `non_wp_rules`
    		#$wp_rewrite->add_external_rule( 'month.php$', $plugin_url );
		#$wp_rewrite->add_external_rule( '^month/calendar/([\w\d-]+)/?', '$plugin_url?d=$1' );
	#}
	#add_action( 'init', 'bjrsu_serviceu_events_calendar_rewrite_rule' );

	function bjrsu_serviceu_events_calendar_parse_request( $request ){

		if( strpos($_SERVER['REQUEST_URI'], 'calendar') !== false ){   

			if( ( isset( $request->query_vars['month'] ) &&
			      isset( $request->query_vars['year'] ) ) ||
			    isset( $request->query_vars['print'] ) ):
				$plugin_url = plugins_url( 'month.php', __FILE__ );
				$plugin_url = substr( $plugin_url, strlen( home_url() ) + 1 );
				include( $plugin_url );
				exit();
			endif;
			if( isset( $request->query_vars['eventid'] ) ):
				$plugin_url = plugins_url( 'event.php', __FILE__ );
				$plugin_url = substr( $plugin_url, strlen( home_url() ) + 1 );
				include( $plugin_url );
				exit();
			endif;

			nocache_headers();  # disable caching of this page
		}
		return $request;
	}
	add_action( 'parse_request', 'bjrsu_serviceu_events_calendar_parse_request' );

	function bjrsu_serviceu_events_calendar_add_query_vars($vars)
	{
		$vars[] = 'eventid';
		$vars[] = 'year';
		$vars[] = 'month';
		$vars[] = 'department';
		$vars[] = 'without';
		$vars[] = 'print';
		#$vp = end($vars);
# print "<!-- DEBUG: bjrsu_serviceu_events_calendar_add_query_vars $vars $vp -->\n";
		return $vars;
	}

	// hook add_query_vars function into query_vars
	add_filter('query_vars', 'bjrsu_serviceu_events_calendar_add_query_vars');

	function bjrsu_serviceu_events_calendar($content)
	{
		global $bjrsu_filter_exclude;
		$bjrsu_filter_exclude = "";
		global $bjrsu_filter_dept;
		$bjrsu_filter_dept = "";
		$bjrsu_filter_dept = get_query_var('department');
#print "<!-- DEBUG: bjrsu_filter_dept $bjrsu_filter_dept -->\n";
		$bjrsu_filter_dept = stripslashes( $bjrsu_filter_dept );
#print "<!-- DEBUG: bjrsu_filter_dept $bjrsu_filter_dept -->\n";
		$bjrsu_filter_exclude = get_query_var('without');
#print "<!-- DEBUG: bjrsu_filter_exclude $bjrsu_filter_exclude -->\n";
		$bjrsu_filter_exclude = stripslashes( $bjrsu_filter_exclude );
#print "<!-- DEBUG: bjrsu_filter_exclude $bjrsu_filter_exclude -->\n";

		$bjrsu_content = $content;
		preg_match_all('!\[serviceu_events_calendar.*?kind=\"(.*?)\"\]!is',$bjrsu_content,$bjrsu_matches);
		$bjrsu_nummatches = count($bjrsu_matches[0]);
		for( $bjrsu_i=0; $bjrsu_i<$bjrsu_nummatches; $bjrsu_i++ )
		{
			$bjrsu_replacement = bjrsu_serviceu_events_calendar_generate($bjrsu_matches[1][$bjrsu_i]);
			$bjrsu_content = preg_replace('/'.preg_quote($bjrsu_matches[0][$bjrsu_i],'/').'/',$bjrsu_replacement,$bjrsu_content,1);
		}

		return $bjrsu_content;
	}
	add_filter( 'the_content', 'bjrsu_serviceu_events_calendar', 9 );


	// bjrsu_date_of_week() takes a date in Ymd and a day of week in 3 letters or more
	// and returns the date of that day. (ie: "sun" or "sunday" would be acceptable values of $day but not "su")

	function bjrsu_date_of_week($Ymd, $day)
	{
		$num = get_option( 'start_of_week' );
		$week_start_day = date_i18n('D', strtotime("Sunday +{$num} days"));
		$timestamp = strtotime($Ymd);
		$start_day_time = strtotime((date_i18n('w',$timestamp)==$num ? "$week_start_day" : "last $week_start_day"), $timestamp);
		$ret_unixtime = strtotime($day,$start_day_time);
		$ret_unixtime = strtotime('+12 hours', $ret_unixtime);
		$ret = date_i18n('Ymd',$ret_unixtime);

		return $ret;
	}


	function bjrsu_serviceu_events_calendar_generate($kind)
	{
		$current_time = time();
		global $bjrsu_html_string;
		$bjrsu_html_string = "";

		global $bjrsu_today;
		$bjrsu_today = date_i18n ("Ymd");

		# Used for parsing the XML from ServiceU
		global $bjrsu_last_event_date;
		global $bjrsu_Occ_id;
		global $bjrsu_Event_id;
		global $bjrsu_Display_times;
		global $bjrsu_Occ_start;
		global $bjrsu_Occ_end;
		global $bjrsu_Event_name;
		global $bjrsu_Event_desc;
		global $bjrsu_Dept;
		global $bjrsu_random_number;
		global $bjrsu_kind;
		global $bjrsu_one_week_from_today;
		global $bjrsu_this_sunday;
		global $bjrsu_one_week_from_sunday;
		$bjrsu_one_week_from_today = date_i18n ("Ymd", strtotime("+7 days"));
		$bjrsu_this_sunday = date_i18n ("Ymd", strtotime("Sunday"));
		$bjrsu_one_week_from_sunday = date_i18n ("Ymd", strtotime("Sunday +7 days"));
		$bjrsu_kind = $kind;

		global $bjrsu_filter_dept;
		global $bjrsu_filter_exclude;

#print "<-- DEBUG: kind $kind before D${bjrsu_filter_dept}D X${bjrsu_filter_exclude}X -->\n";
		if (preg_match('!\+(.*)!',$kind,$reg)) {
			$bjrsu_filter_dept = $reg[1];
		}
		if (preg_match('!\-(.*)!',$kind,$reg)) {
			$bjrsu_filter_exclude = $reg[1];
		}
#print "<-- DEBUG: after D${bjrsu_filter_dept}D X${bjrsu_filter_exclude}X -->\n";
		$bjrsu_deptopt = "";
		if ($bjrsu_filter_dept != "") {
			$bjrsu_deptopt = "&d=" . urlencode($bjrsu_filter_dept);
		}
		if ($bjrsu_filter_exclude != "") {
			$bjrsu_deptopt .= "&x=" . urlencode($bjrsu_filter_exclude);
		}
#print "<-- DEBUG: bjrsu_deptopt $bjrsu_deptopt -->\n";


		#print "<!-- DEBUG: bjrsu_serviceu_events_calendar_generate $kind $bjrsu_one_week_from_today -->\n";

		if ($kind == "thissundaydate") {
			$thissundaydate = date_i18n ("F j, Y", strtotime("Sunday"));
			return $thissundaydate;
		}

		if ($kind == "todaysdate") {
			$thissundaydate = date_i18n ("F j, Y", strtotime("today"));
			return $thissundaydate;
		}

		#$ppath = plugin_dir_url(__FILE__);

		# PHP will read to EOL with fgets...
		$bjrsu_auto_detect_line_endings = $auto_detect_line_endings;
		$auto_detect_line_endings = TRUE;

		##
		$bjrsu_month = date_i18n("F");
		$bjrsu_year = date_i18n("Y");
		$bjrsu_display_month = "$bjrsu_month $bjrsu_year";
		$bjrsu_day = date_i18n("j");
		$bjrsu_mon = date_i18n("n");
		$bjrsu_start_date = sprintf("%d-%02d-%02d",$bjrsu_year,$bjrsu_mon,$bjrsu_day);

		# This prevents us from getting a bad end date
		if ($bjrsu_day > 28)
		{
			$bjrsu_day = 28;
		}
		$bjrsu_next_year = $bjrsu_year;
		$bjrsu_next_mon = $bjrsu_mon + 1;
		if ($bjrsu_next_mon > 12)
		{
			$bjrsu_next_mon = 1;
			$bjrsu_next_year = $bjrsu_next_year + 1;
		}
		$bjrsu_end_date = sprintf("%d-%02d-%02d",$bjrsu_next_year,$bjrsu_next_mon,$bjrsu_day);

		$bjrsu_random_number = rand();

		$org_key = get_option( 'serviceu_calendar_orgkey' );
		$bjrsu_url = "https://api.serviceu.com/rest/events/occurrences?orgKey=$org_key&startDate=$bjrsu_start_date&endDate=$bjrsu_end_date";

		$maint = 1;
		$cache_life = 3600;
		$using_cache = 0;
		$bjrsu_cache_file = get_temp_dir() . "serviceu_cache.$org_key.xml";
		$gopt = stream_context_create (array("ssl" => array("ignore_errors" => true, "verify_peer" => false, "verify_peer_name" => false, "allow_self_signed" => true)));
		set_time_limit(5);
		$handle = fopen ($bjrsu_url, "r", false, $gopt);
		set_time_limit(0);

		$buffer = "";
		$bjrsu_debug = 0;

		if ($handle)
		{
			$now = time();
			$sarray = @stat($bjrsu_cache_file);
			if ($sarray)
			{
				$mtime = $sarray[9]; # modified time
			}
			else
			{
				$mtime = $now - $cache_life;
			}
			$tdiff = $now - $mtime;
			if ($bjrsu_debug)
			{
				print "Tdiff = $tdiff<p>\n";
			}
			if ($tdiff >= $cache_life)
			{
				if ($bjrsu_debug)
				{
					print "check for lock file<p>\n";
				}
				if (!file_exists("$bjrsu_cache_file.lock"))
				{
					if ($bjrsu_debug)
					{
						print "no lock file<p>\n";
					}
					$lhandle = @fopen("$bjrsu_cache_file.lock","w");
					if ($lhandle)
					{
						fwrite($lhandle,$now);
						fclose($lhandle);
						if ($bjrsu_debug)
						{
							print "wrote $now to lock file<p>\n";
						}
						usleep(rand(5,70));
						$lhandle = @fopen("$bjrsu_cache_file.lock","r");
						if ($lhandle)
						{
							$check = fgets($lhandle);
							fclose($lhandle);
							if ($bjrsu_debug)
							{
								print "read $check from lock file<p>\n";
							}
							if ($check == $now)
							{
								$ohandle = @fopen("$bjrsu_cache_file","w");
								if ($ohandle)
								{
									fwrite($ohandle,$buffer);
									#print "<!-- updating cache $ohandle -->\n";
								}
							}
						}
					}
				}
			}
		}
		else
		{
			$handle = @fopen("$bjrsu_cache_file","r");
			$using_cache = 1;
			#print "<!-- using cache $handle -->\n";
		}


		$mysow = get_option( 'start_of_week' );
		$first_of_next_month = mktime(0,0,0,$bjrsu_next_mon,1,$bjrsu_next_year);
		$next_month = date_i18n("F",$first_of_next_month);
		$bjrsu_display_next_month = "$next_month $bjrsu_next_year";

		# used in the mini-calendar headings...

		$daysofweekreallyshort_lang[0] = "S";
		$daysofweekreallyshort_lang[1] = "M";
		$daysofweekreallyshort_lang[2] = "T";
		$daysofweekreallyshort_lang[3] = "W";
		$daysofweekreallyshort_lang[4] = "T";
		$daysofweekreallyshort_lang[5] = "F";
		$daysofweekreallyshort_lang[6] = "S";

		################################

		$bjrsu_html_string .= "<style>\n.monthname { font-size: 1.6em; text-align: center; } ..minicalendar.table, .minicalendar.td, .minicalendar.th { margin-bottom:1px; } @media (max-width: 740px) { .minicalendar { font-size:0.5em; padding:0px; } .minicalendar.table, .minicalendar.td, .minicalendar.th { padding:1px; }  }\n</style>\n";
		$bjrsu_html_string .= "<script language=\"javascript\">\nfunction popwindow(url)\n{\nday = new Date();\nid = day.getTime();\neval(\"page\" + id + \" = window.open(url, '\" + id + \"', 'toolbar=0,scrollbars=1,location=0,statusbar=0,menubar=0,resizable=1,width=700,height=575');\");\n}\n</script>\n";

if ($kind == "all")
{

		$bjrsu_html_string .= '<div class="minicalendar"><center><table width="100%" border="0" cellpadding="10" cellspacing="10"><tr><td width="50%" valign="top"><table width="100%" border="0" cellpadding="0" cellspacing="0"><tr><td align="left" valign="top" width="1">&nbsp;</td><td class="monthname" align="center">';

		if ($using_cache)
		{
			$bjrsu_html_string .= "\t<b>$bjrsu_display_month</b>";
		}
		else
		{
			#$bjrsu_html_string .= "\t<a target=\"_blank\" href=\"https://public.serviceu.com/Calendar/?startMonth=$bjrsu_mon&startYear=$bjrsu_year&deptId=0&canSearch=True&dontShowOpenReg=True&dontShowSubscription=True&includeResources=False&ResourceListAsLocation=False&orgkey=$org_key&RN=$bjrsu_random_number\">$bjrsu_display_month</a>";
			$bjrsu_html_string .= "\t<a target=\"_blank\" href=\"?month=$bjrsu_mon&year=$bjrsu_year$bjrsu_deptopt\">$bjrsu_display_month</a>";
		}

		$bjrsu_html_string .= '</td><td align="right" valign="top" width="1">&nbsp;</td></tr><tr><td colspan="3" bgcolor="#FFFFFF" align="center"><table width="70%" border="0" cellspacing="1" cellpadding="1" bgcolor="#FFFFFF"><tr>';

		$num = get_option( 'start_of_week' );
		$week_start_day = date_i18n('D', strtotime("Sunday +{$num} days"));
		$start_day = strtotime($week_start_day);
		for ($i=0; $i<7; $i++)
		{
			$day_num = date_i18n("w", $start_day);
			$day = $daysofweekreallyshort_lang[$day_num];
			$bjrsu_html_string .= "<td width=\"14%\" align=\"right\">$day</td>";
			$start_day = strtotime("+1 day", $start_day); 
		}
		$bjrsu_html_string .= '</tr>';

		$minical_month = date_i18n("m", $current_time);
		$minical_year = date_i18n("Y", $current_time);
		$first_of_month = $minical_year.$minical_month."01";
		$mydummydow = bjrsu_date_of_week($first_of_month, $week_start_day);
		$start_day = strtotime(bjrsu_date_of_week($first_of_month, $week_start_day));

		$i = 0;
		$whole_month = TRUE;
		do
		{
			$day = date_i18n ("j", $start_day);
			$daylink = date_i18n ("Ymd", $start_day);
			$check_month = date_i18n ("m", $start_day);
			if ($i == 0) $bjrsu_html_string .= "<tr>";
			if ($bjrsu_today == $daylink)
			{
				$bjrsu_html_string .= "<td bgcolor=\"#ddddff\" align=\"right\">";
			}
			else
			{
				$bjrsu_html_string .= "<td align=\"right\">";
			}
			if ($check_month != $minical_month)
			{
				$bjrsu_html_string .= "&nbsp;";
			}
			else
			{
				$bjrsu_html_string .= "$day";
			}
			$bjrsu_html_string .= "</td>";
			$start_day = ($start_day + (24.5 * 60 * 60));
			$i++;
			if ($i == 7)
			{ 
				$bjrsu_html_string .= "</tr>\n";
				$i = 0;
				$checkagain = date_i18n ("m", $start_day);
				if ($checkagain != $minical_month) $whole_month = FALSE;	
			}
		}
		while ($whole_month == TRUE);

		$bjrsu_html_string .= '</table></td> </tr> </table> </td> <td width="50%" valign="top"> <table width="100%" border="0" cellpadding="0" cellspacing="0"> <tr> <td align="left" valign="top" width="1">&nbsp;</td> <td class="monthname" align="center"> ';

		if ($using_cache)
		{
			$bjrsu_html_string .= "\t<b>$bjrsu_display_next_month</b>";
		}
		else
		{
			#$bjrsu_html_string .= "\t<a target=\"_blank\" href=\"https://public.serviceu.com/Calendar/?startMonth=$bjrsu_next_mon&startYear=$bjrsu_next_year&deptId=0&canSearch=True&dontShowOpenReg=True&dontShowSubscription=True&includeResources=False&ResourceListAsLocation=False&orgkey=$org_key&RN=$bjrsu_random_number\">$bjrsu_display_next_month</a>";
			$bjrsu_html_string .= "\t<a target=\"_blank\" href=\"?month=$bjrsu_next_mon&year=$bjrsu_next_year$bjrsu_deptopt\">$bjrsu_display_next_month</a>";
		}

		$bjrsu_html_string .= ' </td> <td align="right" valign="top" width="1">&nbsp;</td> </tr> <tr> <td colspan="3" bgcolor="#FFFFFF" align="center"> <table width="70%" border="0" cellspacing="3" cellpadding="3" bgcolor="#FFFFFF"> <tr> ';

		$start_day = strtotime($week_start_day);
		for ($i=0; $i<7; $i++)
		{
			$day_num = date_i18n("w", $start_day);
			$day = $daysofweekreallyshort_lang[$day_num];
			$bjrsu_html_string .= "<td width=\"14%\" align=\"right\">$day</td>";
			$start_day = strtotime("+1 day", $start_day); 
		}
		$bjrsu_html_string .= '</tr>';

		$minical_time = strtotime("+1 month", $current_time);
		$minical_month = date_i18n("m", $minical_time);
		$minical_year = date_i18n("Y", $minical_time);
		$first_of_month = $minical_year.$minical_month."01";
		$start_day = strtotime(bjrsu_date_of_week($first_of_month, $week_start_day));
		$i = 0;
		$whole_month = TRUE;
		do
		{
			$day = date_i18n ("j", $start_day);
			$daylink = date_i18n ("Ymd", $start_day);
			$check_month = date_i18n ("m", $start_day);
			if ($i == 0) $bjrsu_html_string .= "<tr>";
			$bjrsu_html_string .= "<td align=\"right\">";
			if ($check_month != $minical_month)
			{
				$bjrsu_html_string .= "&nbsp;";
			}
			else
			{
				$bjrsu_html_string .= "$day";
			}
			$bjrsu_html_string .= "</td>";
			$start_day = ($start_day + (24.5 * 60 * 60));
			$i++;
			if ($i == 7)
			{ 
				$bjrsu_html_string .= "</tr>\n";
				$i = 0;
				$checkagain = date_i18n ("m", $start_day);
				if ($checkagain != $minical_month) $whole_month = FALSE;	
			}
		}
		while ($whole_month == TRUE);
		$bjrsu_html_string .= '</table></td></tr></table></td></tr></table></center></div>';
} // mini-months

		global $bjrsu_filter_dept;
		global $bjrsu_filter_exclude;
		if ($bjrsu_filter_dept != "" && $kind == "all") {
			$bjrsu_html_string .= "<div style=\"text-align:center;\"><u>$bjrsu_filter_dept</u><br><a href=\"./\">See all</a></div>";
		}
		if ($bjrsu_filter_exclude != "" && $kind == "all") {
			$bjrsu_html_string .= "<div class=\"show-without-blurb\" style=\"font-size:0.7em; text-align:center;\"><i>(without $bjrsu_filter_exclude)</i><br><a href=\"./\">See all</a></div>";
		}

		if ($kind == "forsunday" || preg_match('![+-](.*)!',$kind,$reg))
		{
			$bjrsu_html_string .= '<div style="width:100%;">';
		}
		else
		{
			$bjrsu_html_string .= '<table class="serviceu-calendar-table" border="0">';
		}

		if ($handle)
		{

			// Create an xml parser
			$xmlParser = xml_parser_create();

			// Set up element handler
			xml_set_element_handler( $xmlParser, "bjrsu_start_element", "bjrsu_end_element" );

			// Set up character handler
			xml_set_character_data_handler( $xmlParser, "bjrsu_char_element" );
			while (!feof ($handle))
			{
				$buffer = fgets($handle);
				if ($ohandle)
				{
					fwrite($ohandle,$buffer);
				}
				if ($buffer !== "")
				{
					$maint = 0;
					xml_parse( $xmlParser, $buffer, feof($handle) );
				}
			}
			fclose ($handle);

			// Free xml parser from memory
			xml_parser_free( $xmlParser );
		} 

		if ($kind == "forsunday" || preg_match('![+-](.*)!',$kind,$reg))
		{
			$bjrsu_html_string .= '</div>';
		}
		else
		{
			$bjrsu_html_string .= '</table>';
		}
#BJR
		if (preg_match('!\+(.*)!',$kind,$reg)) {
			$bjrsu_html_string .= "<div style=\"margin-top:5px; clear:both; width:100%; text-align:center;\"><a target=\"_blank\" href=\"?month=$bjrsu_mon&year=$bjrsu_year$bjrsu_deptopt\">Monthly $bjrsu_filter_dept Calendar</a></div>";
		}
		if (preg_match('!\-(.*)!',$kind,$reg)) {
			$bjrsu_html_string .= "<div style=\"margin-top:5px; clear:both; width:100%; text-align:center;\"><a target=\"_blank\" href=\"?month=$bjrsu_mon&year=$bjrsu_year$bjrsu_deptopt\">Monthly Calendar</a><div class=\"show-without-blurb\" style=\"line-height:0.8em; font-size:0.7em;\">(without $bjrsu_filter_exclude)</div></div>";
		}

		$auto_detect_line_endings = $bjrsu_auto_detect_line_endings;
		return "<div>$bjrsu_html_string</div>";
	}


	function bjrsu_start_element( $parser, $tagName, $attrs )
	{
		global $bjrsu_Occ_id;
		global $bjrsu_Event_id;
		global $bjrsu_Display_times;
		global $bjrsu_Occ_start;
		global $bjrsu_Occ_end;
		global $bjrsu_Event_name;
		global $bjrsu_Event_desc;
		global $bjrsu_Dept;

		global $bjrsu_Text;
		$bjrsu_Text = "";
		switch( $tagName )
		{
			case 'OCCURRENCE':
				$bjrsu_Occ_id = "";
				$bjrsu_Event_id = "";
				$bjrsu_Display_times = "";
				$bjrsu_Occ_start = "";
				$bjrsu_Occ_end = "";
				$bjrsu_Event_name = "";
				$bjrsu_Event_desc = "";
				$bjrsu_Dept = "";
				break;
			# end switch
		}
	}

	function bjrsu_end_element( $parser, $tagName )
	{
		global $bjrsu_html_string;
		global $bjrsu_Occ_id;
		global $bjrsu_Event_id;
		global $bjrsu_Display_times;
		global $bjrsu_Occ_start;
		global $bjrsu_Occ_end;
		global $bjrsu_Event_name;
		global $bjrsu_Event_desc;
		global $bjrsu_Dept;
		global $bjrsu_today;
		global $bjrsu_last_event_date;
		global $bjrsu_random_number;
		global $bjrsu_kind;
		global $bjrsu_one_week_from_today;
		global $bjrsu_this_sunday;
		global $bjrsu_one_week_from_sunday;
		global $bjrsu_Text;
		global $bjrsu_filter_dept;
		global $bjrsu_filter_exclude;
		switch( $tagName )
		{
			case 'OCCURRENCEID':
				$bjrsu_Occ_id = $bjrsu_Text;
				break;
			case 'EVENTID':
				$bjrsu_Event_id = $bjrsu_Text;
				break;
			case 'DISPLAYTIMES':
				$bjrsu_Display_times = $bjrsu_Text;
				break;
			case 'OCCURRENCESTARTTIME':
				$bjrsu_Occ_start = $bjrsu_Text;
				break;
			case 'OCCURRENCEENDTIME':
				$bjrsu_Occ_end = $bjrsu_Text;
				break;
			case 'DEPARTMENTNAME':
				$bjrsu_Dept = $bjrsu_Text;
				break;
			case 'NAME':
				$bjrsu_Event_name = $bjrsu_Text;
				break;
			case 'DESCRIPTION':
				$bjrsu_Event_desc = preg_replace("/http:\/\/[^<>[:space:]]+[[:alnum:]\/]/i",
				"<a target=\"_blank\" href=\"\\0\">\\0</a>", $bjrsu_Text);
				break;
			case 'OCCURRENCE':
				preg_match ("/([0-9]{2})\/([0-9]{2})\/([0-9]{4}) ([0-9]{2}):([0-9]{2}):([0-9]{2}) ([AP]M)/", $bjrsu_Occ_start, $regs);
				if ($regs[7] == 'PM' && $regs[4] < 12) { $regs[4] += 12; }
				if ($regs[7] == 'AM' && $regs[4] == 12) { $regs[4] = 0; }
				$start_unixtime = mktime($regs[4], $regs[5], $regs[6], $regs[1], $regs[2], $regs[3]);
				$start_date = date_i18n("l, F j, Y",$start_unixtime);
				$start_time = date_i18n("g:i A",$start_unixtime);
				$event_date = date_i18n ("Ymd",$start_unixtime);
				if ($event_date >= $bjrsu_today)
				{
#print "<!-- DEBUG: ed $event_date gb $bjrsu_one_week_from_today ED $bjrsu_Dept En $bjrsu_Event_name Ed $bjrsu_Event_desc -->\n";
#print "<-- DEBUG: check D${bjrsu_filter_dept}D X${bjrsu_filter_exclude}X -->\n";
					if ($bjrsu_kind == 'forsunday')
					{
						if ($event_date < $bjrsu_this_sunday ||
						    $event_date > $bjrsu_one_week_from_sunday ||
						    $bjrsu_Dept == "Community Events")
						{
#print "<!-- DEBUG: skipping -->\n";
							break;
						}
					}
					if ($bjrsu_kind == 'weekly')
					{
						if ($event_date > $bjrsu_one_week_from_today)
						{
#print "<!-- DEBUG: skipping -->\n";
							break;
						}
					}

#print "<!-- DEBUG: bjrsu_filter_exclude $bjrsu_filter_exclude Dept $bjrsu_Dept -->\n";
					if ($bjrsu_filter_dept != "" && strstr($bjrsu_Dept,$bjrsu_filter_dept) == FALSE)
					{
						break;
					}
					if ($bjrsu_filter_exclude != "" && strstr($bjrsu_Dept,$bjrsu_filter_exclude) != FALSE)
					{
						break;
					}
					if ($event_date == $bjrsu_today)
					{
						$color = "bgcolor=\"#ddddff\"";
					}
					else
					{
						$color = "";
					}
					preg_match ("/([0-9]{2})\/([0-9]{2})\/([0-9]{4}) ([0-9]{2}):([0-9]{2}):([0-9]{2}) ([AP]M)/", $bjrsu_Occ_end, $regs);
					if ($regs[7] == 'PM' && $regs[4] < 12) { $regs[4] += 12; }
					if ($regs[7] == 'AM' && $regs[4] == 12) { $regs[4] = 0; }
					$end_unixtime = mktime($regs[4], $regs[5], $regs[6], $regs[1], $regs[2], $regs[3]);
					$end_time = date_i18n("g:i A",$end_unixtime);
					$event_end_date = date_i18n ("Ymd",$end_unixtime);
#print "<!-- DEBUG: 1bjrsu_kind $bjrsu_kind display_times $display_times -->\n";
					if ($start_time == "12:00 AM")
					{
						if ($end_time == "11:59 PM")
						{
							$display_times = "All&nbsp;Day ";
#print "<!-- DEBUG: 2bjrsu_kind $bjrsu_kind display_times $display_times -->\n";
						}
						else
						{
							$display_times = "Ends&nbsp;$end_time ";
#print "<!-- DEBUG: 3bjrsu_kind $bjrsu_kind display_times $display_times -->\n";
						}
					}
					else
					{
						if ($end_time == "11:59 PM")
						{
							$display_times = "Starts&nbsp;$start_time ";
#print "<!-- DEBUG: 4bjrsu_kind $bjrsu_kind display_times $display_times -->\n";
						}
						else
						{
							$display_times = "$start_time - $end_time";
#print "<!-- DEBUG: 5bjrsu_kind $bjrsu_kind display_times $display_times -->\n";
						}
					}
#print "<!-- DEBUG: bjrsu_kind $bjrsu_kind display_times $display_times -->\n";
					# when event name is in ALL CAPS then do not show display times...
					# MERRY CHRISTMAS
					#if ($bjrsu_Event_name == strtoupper($bjrsu_Event_name))
					if ($bjrsu_Display_times == "false")
					{
						$display_times = "&nbsp;";
					}
#print "<!-- DEBUG: bjrsu_kind $bjrsu_kind display_times $display_times -->\n";

					if ($bjrsu_kind == "forsunday" || preg_match('![+-](.*)!',$bjrsu_kind,$reg))
					{
						$bjrsu_html_string .= "<div style=\"clear:both;width:100%;\">";
						if ($bjrsu_last_event_date != $start_date)
						{
							$bjrsu_html_string .= "<div style=\"padding-top:0.3em; display:block; clear:both;\"><i>$start_date</i></div>\n";
						}
						$bjrsu_html_string .= "<div style=\"display:block; float:right;\">";
						#$display_times = preg_replace("/:00/","",$display_times);
						#$display_times = preg_replace("/(.*)AM(.*)(AM)/","$1$2$3",$display_times);
						#$display_times = preg_replace("/(.*)PM(.*)(PM)/","$1$2$3",$display_times);
						$display_times = preg_replace("/-.*/","",$display_times);
						$display_times = preg_replace("/ PM/","pm",$display_times);
						$display_times = preg_replace("/ AM/","am",$display_times);
						#$display_times = preg_replace("/:00/","",$display_times);
					}
					else
					{
						$display_times = preg_replace("/ PM/","pm",$display_times);
						$display_times = preg_replace("/ AM/","am",$display_times);
						if ($bjrsu_last_event_date != $start_date)
						{
							$bjrsu_html_string .= "<tr class=\"serviceu-calendar-date\"><td colspan=\"2\">$start_date</td></tr>\n";
						}
						$bjrsu_html_string .= "<tr><td $color valign=\"top\" width=\"170\" nowrap>";
					}
					$bjrsu_last_event_date = $start_date;
					$dt = preg_replace("/ /","&nbsp;",$display_times);
					$bjrsu_html_string .= "&nbsp;$dt";
					if ($bjrsu_kind == "forsunday" || preg_match('![+-](.*)!',$bjrsu_kind,$reg))
					{
						$bjrsu_html_string .= "</div><div style=\"display:block; text-align:left; max-width:68%; padding-left:0.5em; float:left;\">";
					}
					else
					{
						$bjrsu_html_string .= "</td>";
						$bjrsu_html_string .= "<td $color valign=\"top\">";
					}
					if ($using_cache)
					{
						$bjrsu_html_string .= "<b>$bjrsu_Event_name</b><br>$bjrsu_Event_desc";
					}
					else
					{
						$org_key = get_option( 'serviceu_calendar_orgkey' );
						#$bjrsu_html_string .= "<a href=\"javascript:popwindow('https://public.serviceu.com/Calendar/EventDetails?orgKey=$org_key&occId=$bjrsu_Occ_id&deptId=0&dontShowOpenReg=True&RN=$bjrsu_random_number');\">$bjrsu_Event_name</a><br>$bjrsu_Event_desc";
						#$ppath = plugin_dir_url(__FILE__);

						if ($bjrsu_kind == 'forsunday')
						{
							$bjrsu_html_string .= "$bjrsu_Event_name";
						}
						else
						{
							$bjrsu_html_string .= "<a href=\"javascript:popwindow('?eventid=$bjrsu_Occ_id');\">$bjrsu_Event_name</a><br>$bjrsu_Event_desc";
							#$bjrsu_html_string .= "<a href=\"javascript:popwindow('/calendar/event.php?id=$bjrsu_Occ_id');\">$bjrsu_Event_name</a><br>$bjrsu_Event_desc";
						}
					}
					if ($bjrsu_kind == "forsunday" || preg_match('![+-](.*)!',$bjrsu_kind,$reg))
					{
						$bjrsu_html_string .= "</div></div>";
					}
					else
					{
						$bjrsu_html_string .= "</td></tr>\n";
					}
				}
				break;
			# end switch
		}
	}

	function bjrsu_char_element( $parser, $text )
	{
		global $bjrsu_Text;

		$bjrsu_Text .= trim( $text );
	}

	function bjrsu_serviceu_events_calendar_register_settings()
	{
		add_option( 'serviceu_calendar_orgkey', 'ServiceU Organization Key.');
		register_setting('serviceu_events_calendar-option_group', 'serviceu_calendar_orgkey');
	}
	add_action( 'admin_init', 'bjrsu_serviceu_events_calendar_register_settings' );

	function bjrsu_serviceu_events_calendar_register_options_page()
	{
		add_options_page('ServiceU Events Calendar', 'ServiceU Calendar', 'manage_options', 'serviceu_events_calendar', 'bjrsu_serviceu_events_calendar_options_page');
	} 
	add_action( 'admin_menu', 'bjrsu_serviceu_events_calendar_register_options_page' );

        function bjrsu_serviceu_events_calendar_options_page()
	{
?>
<div class="wrap">
<h1>ServiceU Events Calendar Plugin Options</h1>
<form method="post" action="options.php">
<?php
		settings_fields('serviceu_events_calendar-option_group');
?>
<table>
<tr valign="top">
<th scope="row"><label for="serviceu_calendar_orgkey">ServiceU Organization Key</label></th>
<td><input size="50" type="text" id="serviceu_calendar_orgkey" name="serviceu_calendar_orgkey" value="<?php echo get_option('serviceu_calendar_orgkey'); ?>" /></td>
</tr>
</table>
<?php
		submit_button();
?>
</form>
</div>
<?php
	}

	// Add settings link on plugin page
	function bjrsu_serviceu_events_calendar_settings_link($links)
	{ 
		$settings_link = '<a href="options-general.php?page=serviceu_events_calendar">Settings</a>'; 
		array_unshift($links, $settings_link); 
		return $links; 
	}
 
	$plugin = plugin_basename(__FILE__); 
	add_filter("plugin_action_links_$plugin", 'bjrsu_serviceu_events_calendar_settings_link' );

} # if( ! function_exists('bjrsu_serviceu_events_calendar') )

?>
